package uy.edu.fing.chesstrack.modulovision.imgproc;

import org.opencv.core.CvType;
import org.opencv.core.Mat;
import org.opencv.core.MatOfPoint2f;
import org.opencv.core.Point;
import org.opencv.core.Rect;
import org.opencv.core.Size;
import org.opencv.imgproc.Imgproc;

import uy.edu.fing.chesstrack.modulovision.Calibracion;
import android.util.Log;

public class Homografia implements ImgProcInterface {

	private static final String TAG = "CHESSTRACK::Homografia";

	private final  Mat _matrizTransformada;
	private final Rect _rectROI;
	private int _media;
	private int _offset;

	public Homografia() throws Exception {
		super();
		MatOfPoint2f vertices = Calibracion.getInstance().getVertices();

		if (vertices != null){
			Mat src = new Mat(4,1,CvType.CV_32FC2);
			Mat dst = new Mat(4,1,CvType.CV_32FC2);
			
			Point p1, p2, p3, p4;
			Point[] ret = getBestPoints(vertices, Calibracion.getInstance().get_sizeFrame());

			Log.i(TAG, "Best Points= " + ret.toString());
			p1 = ret[0];
			p2 = ret[1];
			p3 = ret[2];
			p4 = ret[3];
			
			// se obtienen las distancias de cada una de las aristas que determinan los vertices 
			// internos mas externos y se determina el promedio de esa medida
			double d12 = Math.sqrt(Math.pow((p1.x- p2.x),2) + Math.pow((p1.y- p2.y),2));
			double d23 = Math.sqrt(Math.pow((p2.x- p3.x),2) + Math.pow((p2.y- p3.y),2));
			double d34 = Math.sqrt(Math.pow((p3.x- p4.x),2) + Math.pow((p3.y- p4.y),2));
			double d14 = Math.sqrt(Math.pow((p4.x- p1.x),2) + Math.pow((p4.y- p1.y),2));

			_media = (int) Math.floor( Math.round((d12+d23+d34+d14)/4));
			// El offset es el tamaño de dos celdas (para abarcar el primer casillero y dejar un borde al tablero
			_offset = 2*(_media/6);
			
			Log.i(TAG, "media= " + _media);
			Log.i(TAG, "offset= " + _offset);

			src.put(0,0, (int)p1.x,(int)p1.y, (int)p2.x,(int)p2.y, (int)p3.x,(int)p3.y, (int)p4.x,(int)p4.y);
			dst.put(0,0, _offset,_offset, _offset+_media,_offset, _offset+_media,_offset+_media, _offset,_offset+_media );

			_rectROI = new Rect(new Point(0,0), new Point(2*_offset+_media,2*_offset+_media));
			Calibracion.getInstance();
			Calibracion.setRectROI(_rectROI);
			
			Log.i(TAG, "rectROI size= " + _rectROI.size());

			_matrizTransformada = Imgproc.getPerspectiveTransform(src,dst);
		} else {
			throw new Exception(TAG + "No se han calculado los vertices");
		}
	}

	/**
	 * Hace homografia y recorta la imagen
	 */
	@Override
	public Mat procesarImagen(Mat inputFrame) {
		Log.i(TAG, "processing Frame - INI");

		// hace la homografia con la matriz calculada en un ppio
		Imgproc.warpPerspective(inputFrame,inputFrame, _matrizTransformada, inputFrame.size());
		Mat tmp = Mat.zeros(_rectROI.size(), CvType.CV_8UC4);
		(inputFrame.submat(_rectROI)).copyTo(tmp);

		Log.i(TAG, "processing Frame - FIN");
		return tmp;
	}

	private Point[] getBestPoints(MatOfPoint2f vertices, Size tamanio){
		double d, best_dist;
		Point[] ret = new Point[4];
		Point[] vect = new Point[4];

		vect[0] = new Point(0,0);
		vect[1] = new Point(tamanio.width,0);
		vect[2] = new Point(tamanio.width,tamanio.height);
		vect[3] = new Point(0,tamanio.height);

		for (int i = 0; i < 4; i++){
			Point aux = vect[i];
			best_dist = Double.MAX_VALUE;
			for(Point pt : vertices.toList()){
				d = Math.sqrt(Math.pow((pt.x- aux.x),2) + Math.pow((pt.y- aux.y),2));
				if (d < best_dist){
					best_dist = d;
					ret[i] = pt;
				}
			}			
		}

		return ret;
	}
}
